import {
  __publicField
} from "../chunk-XEO3YXBM.js";

// bitbazaar/log/log.ts
import { ZoneContextManager } from "@opentelemetry/context-zone";
import { WebTracerProvider } from "@opentelemetry/sdk-trace-web";
import {
  CompositePropagator,
  W3CBaggagePropagator,
  W3CTraceContextPropagator
} from "@opentelemetry/core";
import { DocumentLoadInstrumentation } from "@opentelemetry/instrumentation-document-load";
import { FetchInstrumentation } from "@opentelemetry/instrumentation-fetch";
import { UserInteractionInstrumentation } from "@opentelemetry/instrumentation-user-interaction";
import { Resource } from "@opentelemetry/resources";
import { SemanticResourceAttributes } from "@opentelemetry/semantic-conventions";
import { registerInstrumentations } from "@opentelemetry/instrumentation";
import { BatchSpanProcessor } from "@opentelemetry/sdk-trace-base";
import { BatchLogRecordProcessor, LoggerProvider } from "@opentelemetry/sdk-logs";
import { OTLPLogExporter } from "@opentelemetry/exporter-logs-otlp-http";
import { OTLPMetricExporter } from "@opentelemetry/exporter-metrics-otlp-http";
import { OTLPTraceExporter } from "@opentelemetry/exporter-trace-otlp-http";
import { MeterProvider, PeriodicExportingMetricReader } from "@opentelemetry/sdk-metrics";
var _LOG;
var GlobalLog = class {
  /** Create the global logger, must setup oltp (http), console can be optionally setup and will just print logs. */
  constructor({
    otlp,
    console: console2 = void 0
  }) {
    __publicField(this, "loggerProvider");
    __publicField(this, "logger");
    __publicField(this, "tracerProvider");
    __publicField(this, "tracer");
    __publicField(this, "meterProvider");
    __publicField(this, "console");
    __publicField(this, "oltp");
    /// We override global console functions to do filtering and emit to oltp, need to keep access to the inner ones:
    __publicField(this, "orig_console_fns");
    this.console = console2 ? console2 : null;
    this.oltp = otlp;
    this._set_console_fns();
    const resource = new Resource({
      [SemanticResourceAttributes.SERVICE_NAME]: otlp.service_name,
      [SemanticResourceAttributes.SERVICE_VERSION]: otlp.service_version
    });
    const baseExporterConfig = {
      keepAlive: true
    };
    const bufferConfig = {
      maxQueueSize: 2048,
      maxExportBatchSize: 512,
      scheduledDelayMillis: 5e3,
      exportTimeoutMillis: 3e4
    };
    this.meterProvider = new MeterProvider({
      resource,
      readers: [
        new PeriodicExportingMetricReader({
          exporter: new OTLPMetricExporter({
            ...baseExporterConfig,
            url: `${otlp.endpoint}/v1/metrics`
          }),
          exportIntervalMillis: 6e4
          // Haven't found a way for it to not send when no metrics yet, so changing from 1s to 60s to not bloat the network logs of a client.
        })
      ]
    });
    this.loggerProvider = new LoggerProvider({
      resource
    });
    this.loggerProvider.addLogRecordProcessor(
      new BatchLogRecordProcessor(
        new OTLPLogExporter({
          ...baseExporterConfig,
          url: `${otlp.endpoint}/v1/logs`
        })
      )
    );
    this.logger = this.loggerProvider.getLogger("GlobalLog");
    this.tracerProvider = new WebTracerProvider({ resource });
    this.tracerProvider.addSpanProcessor(
      new BatchSpanProcessor(
        new OTLPTraceExporter({
          ...baseExporterConfig,
          url: `${otlp.endpoint}/v1/traces`
        }),
        bufferConfig
      )
    );
    this.tracerProvider.register({
      contextManager: new ZoneContextManager(),
      // Configure the propagator to enable context propagation between services,
      // uses the W3C Trace Headers (traceparent, tracestate) and W3C Baggage Headers (baggage).
      propagator: new CompositePropagator({
        propagators: [new W3CBaggagePropagator(), new W3CTraceContextPropagator()]
      })
    });
    this.tracer = this.tracerProvider.getTracer("GlobalLog");
    registerInstrumentations({
      instrumentations: [
        // Trace user site interactions:
        new UserInteractionInstrumentation({}),
        // Trace client document loading:
        new DocumentLoadInstrumentation({}),
        // Auto instrument fetch requests:
        new FetchInstrumentation({})
      ]
    });
    _LOG = this;
  }
  /**
   * Get a new Meter instance to record metrics with.
   *
   * @example
   *  ```typescript
   * const meter = globalLog.getMeter("example-meter");
   * const counter = meter.createCounter('metric_name');
   * counter.add(10, { 'key': 'value' });
   * ```
   */
  meter(name, opts = {}) {
    return this.meterProvider.getMeter(
      name,
      typeof opts === "object" && "version" in opts ? opts.version : void 0,
      opts
    );
  }
  /**
   * Run a sync callback inside a span.
   */
  withSpan(name, cb) {
    return this.tracer.startActiveSpan(name, (span) => {
      const result = cb(span);
      span.end();
      return result;
    });
  }
  /**
   * Run an async callback inside a span.
   */
  withSpanAsync(name, cb) {
    return this.tracer.startActiveSpan(name, (span) => {
      return cb(span).then((result) => {
        span.end();
        return result;
      });
    });
  }
  /** Log a debug message. */
  debug(message, ...attrs) {
    this._log_inner("DEBUG", message, ...attrs);
  }
  /** Log an info message. */
  info(message, ...attrs) {
    this._log_inner("INFO", message, ...attrs);
  }
  /** Log a warning message. */
  warn(message, ...attrs) {
    this._log_inner("WARN", message, ...attrs);
  }
  /** Log an error message. */
  error(message, ...attrs) {
    this._log_inner("ERROR", message, ...attrs);
  }
  _joined_msg_and_attrs(message, attrs) {
    if (attrs.length === 0) {
      return message;
    }
    return `${message} ${attrs.join(" ")}`;
  }
  _log_inner(severityText, message, ...attrs) {
    if (this.console) {
      let emit = false;
      let emitter;
      switch (this.console.level_from) {
        case "DEBUG": {
          emit = true;
          emitter = this.orig_console_fns.debug;
          break;
        }
        case "INFO": {
          emit = severityText !== "DEBUG";
          emitter = this.orig_console_fns.info;
          break;
        }
        case "WARN": {
          emit = severityText === "WARN" || severityText === "ERROR";
          emitter = this.orig_console_fns.warn;
          break;
        }
        case "ERROR": {
          emit = severityText === "ERROR";
          emitter = this.orig_console_fns.error;
          break;
        }
      }
      if (emit) {
        if (this.console.custom_out) {
          this.console.custom_out(this._joined_msg_and_attrs(message, attrs));
        } else {
          emitter(message);
        }
      }
    }
    let emitOltp = false;
    switch (this.oltp.level_from) {
      case "DEBUG": {
        emitOltp = true;
        break;
      }
      case "INFO": {
        emitOltp = severityText !== "DEBUG";
        break;
      }
      case "WARN": {
        emitOltp = severityText === "WARN" || severityText === "ERROR";
        break;
      }
      case "ERROR": {
        emitOltp = severityText === "ERROR";
        break;
      }
    }
    if (emitOltp) {
      this.logger.emit({
        severityText,
        body: this._joined_msg_and_attrs(message, attrs)
      });
    }
  }
  _set_console_fns() {
    this.orig_console_fns = {
      log: console.log,
      debug: console.debug,
      info: console.info,
      warn: console.warn,
      error: console.error
    };
    console.log = (msg, ...attrs) => this.info(msg, ...attrs);
    console.debug = (msg, ...attrs) => this.debug(msg, ...attrs);
    console.info = (msg, ...attrs) => this.info(msg, ...attrs);
    console.warn = (msg, ...attrs) => this.warn(msg, ...attrs);
    console.error = (msg, ...attrs) => this.error(msg, ...attrs);
  }
};
var LOG = new Proxy(GlobalLog.prototype, {
  get: (target, property, receiver) => {
    if (_LOG) {
      return Reflect.get(_LOG, property, receiver);
    }
    throw new Error("Global log not yet initialized!");
  }
});
export {
  GlobalLog,
  LOG
};
