import {
  genPath
} from "../chunk-CQILZEMP.js";
import "../chunk-XEO3YXBM.js";

// bitbazaar/vite/createConfig.ts
import preact from "@preact/preset-vite";
import { minify } from "html-minifier-terser";
import { defineConfig } from "vite";
import checker from "vite-plugin-checker";
import Inspect from "vite-plugin-inspect";
import { VitePWA } from "vite-plugin-pwa";
import fs from "fs/promises";

// bitbazaar/vite/genProxy.ts
var genTargetOptions = (target) => {
  return {
    // Make sure DOES NOT end with /: (if it has one it won't forward sub paths)
    target: genPath(target, { eSlash: false }),
    // If true, django csrf breaks I think because csrf comes in as port 3000,
    // but vites rewritten headers as django's port 8000,
    // seems like there aren't any issues in dev with just leaving as vite domain.
    changeOrigin: false
  };
};
var genBackendProxies = ({ target, matches, negate }) => {
  const result = {};
  if (negate) {
    result[`^(?!${matches.join("|")}).*$`] = genTargetOptions(target);
  } else {
    matches.forEach((path) => {
      result[path] = genTargetOptions(target);
    });
  }
  return result;
};

// bitbazaar/vite/createConfig.ts
var baseNonFrontendGlobs = [
  "**/.git/**",
  "**/venv/**",
  "**/.venv/**",
  "**/node_modules/**",
  "**/dist/**",
  "**/lib/**",
  "**/cypress/**",
  "**/coverage/**",
  "**/.eslintrc.*/**",
  "**/*.etch.*",
  "**/.{idea,git,cache,output,temp,mypy_cache,pytype,pytest,pyright}/**",
  "**/{karma,rollup,webpack,vite,vitest,jest,ava,babel,nyc,cypress,tsup,build}.config.*"
];
var pathExists = async (path) => {
  try {
    await fs.access(path);
    return true;
  } catch {
    return false;
  }
};
var createConfig = (mode, conf) => {
  const isProd = mode === "production";
  const isTest = mode === "test";
  const isDev = mode === "development";
  if (!isProd && !isTest && !isDev) {
    throw new Error(`Unexpected vite mode: ${mode}`);
  }
  console.log(`Vite mode: ${mode}, prod=${isProd}, test=${isTest}, dev=${isDev}`);
  const assetsPath = genPath(conf.staticPath, {
    extra: ["dist"]
  });
  const assetsUrl = genPath(conf.staticUrl, {
    extra: ["dist"]
  });
  const nonFrontendGlobs = [...baseNonFrontendGlobs, ...conf.extraNonFrontendGlobs || []];
  const plugins = [
    preact({
      devToolsEnabled: !isProd,
      prefreshEnabled: !isProd
    }),
    // The service worker / pwa stuff:
    VitePWA({
      registerType: "autoUpdate",
      workbox: {
        // Precache everything found in the vite dist folder:
        globDirectory: genPath(assetsPath),
        // Excluding html, we only have one root index.html and want that to always be fresh:
        globPatterns: ["**/*.{js,css,ico,png,svg}"],
        // Don't fallback on document based (e.g. `/some-page`) requests
        // Even though this says `null` by default, I had to set this specifically to `null` to make it work
        navigateFallback: null,
        // Tell the worker it should be finding all the files from the static domain, rather than the backend:
        modifyURLPrefix: {
          "": genPath(assetsUrl)
        }
      },
      // Needs to come from the backend/same domain otherwise CORS won't work (classic worker problems)
      srcDir: genPath(assetsPath),
      outDir: genPath(conf.sameDomStaticPath, {
        extra: ["sworker"]
      }),
      // Note the mainfest.webmanifest is incorrectly placed in static/dist, but the links go the backend url.
      // Bug in the lib, haven't found a proper way to solve.
      // A secondary plugin below moves the file at the end.
      base: genPath(conf.sameDomStaticUrl, {
        extra: ["sworker"]
      }),
      buildBase: genPath(conf.sameDomStaticUrl, {
        extra: ["sworker"]
      }),
      // Puts the sw importer script directly in the index.html head, rather than a separate file:
      injectRegister: "inline",
      // Allowing the service worker to control the full stie rather than the dir it comes from:
      // this requires the Service-Worker-Allowed: "/" header to be passed from the worker serve directory on backend (which it is)
      scope: "/",
      manifest: {
        name: conf.siteName,
        short_name: conf.siteName,
        description: conf.siteDescription,
        theme_color: "#031033",
        background_color: "#031033",
        display: "standalone",
        start_url: "/",
        icons: [
          {
            src: conf.favicon192PngPath,
            sizes: "192x192",
            type: "image/png",
            purpose: "any maskable"
          },
          {
            src: conf.favicon512PngPath,
            sizes: "512x512",
            type: "image/png",
            purpose: "any maskable"
          }
        ]
      }
    }),
    {
      name: "move-webmanifest",
      // the name of your custom plugin. Could be anything.
      apply: "build",
      enforce: "post",
      closeBundle: async () => {
        const oldLoc = genPath(assetsPath, {
          extra: ["manifest.webmanifest"]
        });
        const newLoc = genPath(conf.sameDomStaticPath, {
          extra: ["sworker", "manifest.webmanifest"]
        });
        if (!await pathExists(genPath(conf.sameDomStaticPath, { extra: ["sworker"] }))) {
          await fs.mkdir(genPath(conf.sameDomStaticPath, { extra: ["sworker"] }));
        }
        console.log(`Moving webmanifest from ${oldLoc} to ${newLoc}`);
        await fs.rename(oldLoc, newLoc);
        const indexLoc = genPath(assetsPath, {
          extra: ["index.html"]
        });
        console.log(`Reading index.html from ${oldLoc} and minifying...`);
        const indexSrc = await fs.readFile(indexLoc, "utf-8");
        const indexMinified = await minify(indexSrc, {
          collapseBooleanAttributes: true,
          collapseInlineTagWhitespace: true,
          collapseWhitespace: true,
          useShortDoctype: true,
          removeComments: true,
          minifyCSS: true,
          minifyJS: true
        });
        const newIndexLoc = genPath(conf.sameDomStaticPath, {
          extra: ["site_index.html"]
        });
        console.log(
          `Minified index.html from ${indexSrc.length} to ${indexMinified.length} bytes, writing to ${newIndexLoc}.`
        );
        await fs.writeFile(newIndexLoc, indexMinified);
        await fs.rm(indexLoc);
      }
    }
  ];
  if (isDev && conf.inspect) {
    plugins.push(Inspect());
  }
  if (!isTest) {
    plugins.push(
      checker({
        typescript: true,
        enableBuild: true,
        // Want it to validate type checking on build too
        // Show errors in terminal & UI:
        terminal: true,
        overlay: true
      })
    );
  }
  const config = {
    clearScreen: false,
    // Don't clear screen when running vite commands
    plugins,
    resolve: {
      // Providing absolute paths:
      alias: conf.alias,
      // Fixes esm problem with rrd:
      // react-router-dom specifies "module" field in package.json for ESM entry
      // if it's not mapped, it uses the "main" field which is CommonJS that redirects to CJS preact
      mainFields: ["module"]
    },
    server: {
      // open: true, // Open the web page on start
      host: "localhost",
      port: conf.port,
      watch: {
        // Don't check non frontend files:
        ignored: nonFrontendGlobs
      },
      // Proxy all backend requests to django/fastapi:
      proxy: conf.proxy ? genBackendProxies(conf.proxy) : void 0,
      fs: conf.serverFs
    },
    // When being served from django in production, js internal assets urls need to use url to the js assets inside the static url,
    // rather than in dev where they're just served from the root:
    base: genPath(isProd ? assetsUrl : "/"),
    // The production build config:
    build: {
      outDir: genPath(assetsPath),
      // Disabled by default when outside root, just a precautionary check I'm happy to disable:
      emptyOutDir: true
    },
    define: {
      // Vite doesn't have process, so define the specifically needed ones directly:
      "process.env.NODE_ENV": JSON.stringify(mode)
    },
    // (Attempted!) size and perf optimisations:
    json: {
      stringify: true
    },
    esbuild: {
      exclude: nonFrontendGlobs
    }
  };
  return defineConfig(config);
};
export {
  createConfig
};
